"use strict";

var _prototypeProperties = function (child, staticProps, instanceProps) { if (staticProps) Object.defineProperties(child, staticProps); if (instanceProps) Object.defineProperties(child.prototype, instanceProps); };

var _classCallCheck = function (instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } };

var crypto = require("crypto");

function getHash(vinylFile) {
	return crypto.createHash("md5").update(vinylFile.contents).digest("hex");
}

/**
 * Class that uses a vinyl object as its key.
 *
 * It uses a checksum of the vinyl objects content to determine if the file has changed and thus
 * if the value associated with the object is still valid.
 */
var VinylCache = (function () {
	function VinylCache() {
		_classCallCheck(this, VinylCache);

		this.hashes = {};
		this.cache = {};
	}

	_prototypeProperties(VinylCache, null, {
		get: {
			/**
    * Get a value for a given vinyl file
    *
    * If no value is found or if the hashes don't match null is returned. If the second parameter is
    * set to true then the hash will be ignored and any found value will be returned.
    */
			value: function get(vinylFile, ignoreHash) {
				var cached = this.cache[vinylFile.path];
				if (ignoreHash) {
					return cached || null;
				}
				if (cached) {
					var hash = getHash(vinylFile);
					if (this.hashes[vinylFile.path] === hash) {
						return cached;
					}
				}
				return null;
			},
			writable: true,
			configurable: true
		},
		set: {
			value: function set(vinylFile, content) {
				this.cache[vinylFile.path] = content;
				var hash = getHash(vinylFile);
				this.hashes[vinylFile.path] = hash;
			},
			writable: true,
			configurable: true
		}
	});

	return VinylCache;
})();

module.exports = VinylCache;