"use strict";

//TODO: all sorts of caching (at least to allow cyclic requires)
//TODO: Do base paths in a better way, it's a bit fragile now
//TODO: Check if upwards paths (../) work with vinyl
var _require = require("./f");

var asyncReduce = _require.asyncReduce;
var catcher = _require.catcher;
var path = require("path");
var fs = require("fs");
var vinylFs = require("vinyl-fs");
var debug = require("debug")("yoloader:resolve");
var resolveFile = require("./dependencyResolvers/resolveFile");
var resolveJsFile = require("./dependencyResolvers/resolveJsFile");
var resolvePackageMain = require("./dependencyResolvers/resolvePackageMain");
var resolveFolderIndex = require("./dependencyResolvers/resolveFolderIndex");
var resolveNodeModule = require("./dependencyResolvers/resolveNodeModule");
var resolvePath = require("./dependencyResolvers/resolvePath");

/**
 * Resolves the dependencies for a file to full file paths
 */
module.exports = function dependencyResolver(chunk, instance) {
	instance.resolverCache = instance.resolverCache || {};
	instance.resolverCache[chunk.path] = instance.resolverCache[chunk.path] || {};

	var compileOptions = instance.options;

	debug("Creating dependency resolver for " + chunk.path);
	return function resolveDependency(depName, done) {
		debug("Resolving dependency from " + chunk.path + " to " + depName);

		//TODO: cache promises instead such that fast successive calls for the same file
		//still are only resolved once.
		var cached = instance.resolverCache[chunk.path][depName];
		if (cached) {
			//If we've found the file in cache then it doesn't necessarily mean it still exists.
			//So we'll do an exists check first. If the file exists we can use the cached value,
			//otherwise we'll need to remove it from cache and try again
			return fs.exists(cached.file, function (exists) {
				if (exists) {
					return done(null, cached);
				} else {
					delete instance.resolverCache[chunk.path][depName];
					return resolveDependency(depName, done);
				}
			});
		}
		var dep = {
			from: chunk.path,
			to: depName,
			file: null,
			base: null,
			//The resolved path under which it will be loaded. This can be used to overwrite the normal
			//behaviour, e.g. when you expose a file under a different name than it's actual filename.
			as: null
		};
		asyncReduce(instance.dependencyResolvers, function (result, resolver, index, arr, cb) {
			var opts = {
				compileOptions: compileOptions,
				base: chunk.base,
				resolve: resolveDependency
			};
			resolver(dep, opts, cb);
		}, null, function (err, dep) {
			if (err) {
				done(err);
			} else if (dep.file) {
				instance.resolverCache[chunk.path][depName] = dep;
				done(null, dep);
			} else {
				done(null, false);
			}
		});
	};
};

module.exports.defaultResolvers = [resolveFile, resolveJsFile, resolvePackageMain, resolveFolderIndex, resolveNodeModule, resolvePath];