"use strict";

var _require = require("./util");

var isGlobal = _require.isGlobal;
var isResolved = _require.isResolved;
var debug = require("debug")("yoloader:resolve:file-alias");
var _require2 = require("../f");

var catcher = _require2.catcher;
var resolveFile = require("./resolveFile");

/**
 * Creates a resolver that aliases a file and then tries to resolve that file.
 *
 * @param {Function} transformFunc A function that should created the aliased name. It get's passed
 *                                 a dependency object and should return a new dependency object
 *                                 with new paramters. It should NOT alter the passed in object.
 * @return {Function} A new resolver that will try to resolve the aliased dependency as a file
 */
module.exports = function fileAliasResolver(transformFunc) {
	return function resolveFileAlias(dep, opts, cb) {
		if (isGlobal(dep.to) || isResolved(dep)) {
			return cb(null, dep);
		}
		var onSuccess = catcher(cb);

		var newDep = transformFunc(dep);
		debug("Trying to resolve " + dep.to + " from " + dep.from + " as aliased file " + newDep.to);

		resolveFile(newDep, opts, onSuccess(function (res) {
			dep.file = res.file;
			dep.base = res.base;
			dep.as = res.as;
			cb(null, dep);
		}));
	};
};