"use strict";

var _require = require("./util");

var isGlobal = _require.isGlobal;
var isResolved = _require.isResolved;
var existsAndIsFile = _require.existsAndIsFile;
var fs = require("fs");
var path = require("path");
var debug = require("debug")("yoloader:resolve:package-main");
var _require2 = require("../f");

var catcher = _require2.catcher;


/**
 * Tries to resolve a dependency as a package (via package.json)
 */
module.exports = function resolvePackageMain(dep, opts, cb) {
	if (isGlobal(dep.to) || isResolved(dep)) {
		return cb(null, dep);
	}
	debug("Trying to resolve " + dep.to + " from " + dep.from + " via package.json#main");
	var onSuccess = catcher(cb);

	var folder = path.resolve(path.dirname(dep.from), dep.to);
	var packagePath = path.join(folder, "package.json");

	existsAndIsFile(packagePath, onSuccess(function (exists) {
		if (exists) {
			debug("Found package.json for " + dep.to + " from " + dep.from);

			fs.readFile(packagePath, "utf8", onSuccess(function (packageContent) {
				var main = JSON.parse(packageContent).main;
				if (main && typeof main !== "string") {
					console.warn("package.json found with an invalid main field. Ignoring file while resoling dependencies. File: ", packagePath);
				}
				if (main && typeof main === "string") {
					var mainFile = path.resolve(path.dirname(packagePath), main);
					debug("Found main field " + mainFile + " in package.json for " + dep.to + " from " + dep.from);

					//Now lets recursively try to resolve the found file using the resolving strategy
					opts.resolve(mainFile, onSuccess(function (res) {
						dep.file = res.file;
						dep.base = opts.base;
						dep.as = res.as;
						cb(null, dep);
					}));
				} else {
					cb(null, dep);
				}
			}));
		} else {
			debug("No package.json found for " + dep.to + " from " + dep.from);
			return cb(null, dep);
		}
	}));
};