"use strict";

var through = require("through2");
var path = require("path");
var applySourceMap = require("vinyl-sourcemaps-apply");
var prefixSourceMap = require("./util/prefixSourceMap");

/**
 * Creates the __dirname and __filename variables for a file.
 *
 * @param {String} [opts.base] Base path from which __dirname and __filename will be calculated.
 *                             If no base path is given chunk.base will be used.
 * @param {String} [opts.prefix] path that will be prefixed to __dirname and __filename.
 */
module.exports = function (opts) {
	var pathPrefix = opts.prefix || "/";
	return through.obj(function (chunk, enc, done) {
		var base = opts.base || chunk.base;
		var fileContent = chunk.contents.toString();
		var filePath = path.relative(base, chunk.path);
		filePath = path.resolve(pathPrefix, filePath);
		var dirPath = path.dirname(filePath);
		//Stringify should automatically make sure that the string that comes out of it is quoted
		//and valid javascript
		var filename = JSON.stringify(filePath);
		var dirname = JSON.stringify(dirPath);
		var prefix = "var __dirname=" + dirname + ",__filename=" + filename + ";";
		chunk.contents = new Buffer(prefix + fileContent);
		if (chunk.sourceMap) {
			var map = prefixSourceMap(chunk, prefix, fileContent);
			applySourceMap(chunk, map);
		}
		done(null, chunk);
	});
};